#include "RegExpParser.h"
#include "regexp/Alternation.h"
#include "regexp/Concatenation.h"
#include "regexp/Iteration.h"

RegExpParser::RegExpParser(RegExpLexer& lexer) : m_Lexer(lexer) {

}

void RegExpParser::parse() {
  this->Expression();
  Token token = m_Lexer.token();
  if(token.type != TEOF) throw ParseError();
}

void RegExpParser::Expression() {
  this->Term();
  this->ExpressionCont();
}

void RegExpParser::ExpressionCont() {
  Token token = m_Lexer.token();
  if(token.type == PLUS) {
    m_Lexer.next();
    this->Term();
    this->ExpressionCont();
  } else {
    return;
  }
}

void RegExpParser::Term() {
  this->Factor();
  this->TermCont();
}

void RegExpParser::TermCont() {
  Token token = m_Lexer.token();
  if(token.type == SYMBOL || token.type == LPAR || token.type == EPS || token.type == EMPTY) {
    this->Factor();
    this->TermCont();
  } else {
    return;
  }
}

void RegExpParser::Factor() {
  Token token = m_Lexer.token();
  if(token.type == LPAR) {
    m_Lexer.next();
    this->Expression();
    token = m_Lexer.token();
    if(token.type != RPAR) throw ParseError();
    m_Lexer.next();
    this->Star();
  } else if(token.type == EPS) {
    m_Lexer.next();
    this->Star();
  } else if(token.type == EMPTY) {
    m_Lexer.next();
    this->Star();
  } else if(token.type == SYMBOL) {
    m_Lexer.next();
    this->Star();
  } else {
    throw ParseError();
  }
}

void RegExpParser::Star() {
  Token token = m_Lexer.token();
  if(token.type == STAR) {
    m_Lexer.next();
    this->Star();
  } else {
    return;
  }
}
