/*
 * AutomatonDiff.cpp
 *
 *  Created on: Apr 1, 2013
 *      Author: martin
 */

#include "AutomatonDiff.h"
#include "automaton/State.h"

#include "automaton/Automaton.h"

#include <set>
#include <map>
#include <list>
#include <utility>
#include <vector>
#include <typeinfo>
#include <iostream>
#include <algorithm>

using namespace std;
using namespace automaton;

bool AutomatonDiff::testDiff(const automaton::UnknownAutomaton& a, const automaton::UnknownAutomaton& b) {
	return  a.getBlankSymbol()   == b.getBlankSymbol()   &&
			a.getFinalStates()   == b.getFinalStates()   &&
			a.getInitialStates() == b.getInitialStates() &&
			a.getInputAlphabet() == b.getInputAlphabet() &&
			a.getStackAlphabet() == b.getStackAlphabet() &&
			a.getStartSymbols()  == b.getStartSymbols()  &&
			a.getStates()        == b.getStates()        &&
			a.getTapeAlphabet()  == b.getTapeAlphabet()  &&
			a.getTransitions()   == b.getTransitions()   ;
}

template <class T> void AutomatonDiff::setDiff(const set<T> a, const set<T> b) {
	set<T> aMinusB;
	set_difference(a.begin(), a.end(), b.begin(), b.end(), std::inserter(aMinusB, aMinusB.begin()));

	set<T> bMinusA;
	set_difference(b.begin(), b.end(), a.begin(), a.end(), std::inserter(bMinusA, bMinusA.begin()));

	for(typename set<T>::const_iterator iter = aMinusB.begin(); iter != aMinusB.end(); iter++) {
		cout << "< " << *iter << endl;
	}
	cout << "---" << endl;
	for(typename set<T>::const_iterator iter = bMinusA.begin(); iter != bMinusA.end(); iter++) {
		cout << "> " << *iter << endl;
	}
}

template <class T> void AutomatonDiff::listDiff(const list<T> a, const list<T> b) {
	list<T> aMinusB;
	set_difference(a.begin(), a.end(), b.begin(), b.end(), std::inserter(aMinusB, aMinusB.begin()));

	list<T> bMinusA;
	set_difference(b.begin(), b.end(), a.begin(), a.end(), std::inserter(bMinusA, bMinusA.begin()));

	for(typename list<T>::const_iterator iter = aMinusB.begin(); iter != aMinusB.end(); iter++) {
		cout << "< " << *iter << endl;
	}
	cout << "---" << endl;
	for(typename list<T>::const_iterator iter = bMinusA.begin(); iter != bMinusA.end(); iter++) {
		cout << "> " << *iter << endl;
	}
}

void AutomatonDiff::printDiff(const automaton::UnknownAutomaton& a, const automaton::UnknownAutomaton& b) {
	cout << "AutomataDiffer" << endl;

	if(a.getBlankSymbol() != b.getBlankSymbol()) {
		cout << "Blank symbol" << endl;

		cout << "< " << a.getBlankSymbol() << endl;
		cout << "---" << endl;
		cout << "> " << b.getBlankSymbol() << endl;
	}

	if(a.getFinalStates() != b.getFinalStates()){
		cout << "FinalStates" << endl;

		AutomatonDiff::setDiff(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialStates() != b.getInitialStates()){
		cout << "InitialStates" << endl;

		AutomatonDiff::setDiff(a.getInitialStates(), b.getInitialStates());
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		cout << "InputAlphabet" << endl;

		AutomatonDiff::setDiff(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStackAlphabet() != b.getStackAlphabet()) {
		cout << "StackAlphabet" << endl;

		AutomatonDiff::setDiff(a.getStackAlphabet(), b.getStackAlphabet());
	}

	if(a.getStartSymbols() != b.getStartSymbols()) {
		cout << "StartSymbols" << endl;

		AutomatonDiff::listDiff(a.getStartSymbols(), b.getStartSymbols());
	}

	if(a.getStates() != b.getStates()) {
		cout << "States" << endl;

		AutomatonDiff::setDiff(a.getStates(), b.getStates());
	}

	if(a.getTapeAlphabet() != b.getTapeAlphabet()) {
		cout << "TapeAlphabet" << endl;

		AutomatonDiff::setDiff(a.getTapeAlphabet(), b.getTapeAlphabet());
	}

	if(a.getTransitions() != b.getTransitions()) {
		cout << "Transitions" << endl;

		AutomatonDiff::setDiff(a.getTransitions(), b.getTransitions());
	}
}

void AutomatonDiff::diff(const automaton::UnknownAutomaton& a, const automaton::UnknownAutomaton& b) {
	if(!AutomatonDiff::testDiff(a, b)) AutomatonDiff::printDiff(a, b);
}
