/*
 * GrammarDiff.cpp
 *
 *  Created on: Apr 1, 2013
 *      Author: honza
 */

#include "GrammarDiff.h"

#include <set>
#include <map>
#include <list>
#include <utility>
#include <vector>
#include <typeinfo>
#include <iostream>
#include <algorithm>

using namespace std;
using namespace grammar;

bool GrammarDiff::testDiff(const grammar::UnknownGrammar& a, const grammar::UnknownGrammar& b) {
	return  a.getNonTerminalSymbols() == b.getNonTerminalSymbols() &&
			a.getRules()              == b.getRules()              &&
			a.getStartSymbol()        == b.getStartSymbol()        &&
			a.getTerminalSymbols()    == b.getTerminalSymbols()    ;
}

template <class T> void GrammarDiff::setDiff(const set<T> a, const set<T> b) {
	set<T> aMinusB;
	set_difference(a.begin(), a.end(), b.begin(), b.end(), std::inserter(aMinusB, aMinusB.begin()));

	set<T> bMinusA;
	set_difference(b.begin(), b.end(), a.begin(), a.end(), std::inserter(bMinusA, bMinusA.begin()));

	for(typename set<T>::const_iterator iter = aMinusB.begin(); iter != aMinusB.end(); iter++) {
		cout << "< " << *iter << endl;
	}
	cout << "---" << endl;
	for(typename set<T>::const_iterator iter = bMinusA.begin(); iter != bMinusA.end(); iter++) {
		cout << "> " << *iter << endl;
	}
}

template <class T> void GrammarDiff::listDiff(const list<T> a, const list<T> b) {
	list<T> aMinusB;
	set_difference(a.begin(), a.end(), b.begin(), b.end(), std::inserter(aMinusB, aMinusB.begin()));

	list<T> bMinusA;
	set_difference(b.begin(), b.end(), a.begin(), a.end(), std::inserter(bMinusA, bMinusA.begin()));

	for(typename list<T>::const_iterator iter = aMinusB.begin(); iter != aMinusB.end(); iter++) {
		cout << "< " << *iter << endl;
	}
	cout << "---" << endl;
	for(typename list<T>::const_iterator iter = bMinusA.begin(); iter != bMinusA.end(); iter++) {
		cout << "> " << *iter << endl;
	}
}

void GrammarDiff::printDiff(const grammar::UnknownGrammar& a, const grammar::UnknownGrammar& b) {
	cout << "GrammarDiffer" << endl;

	if(a.getNonTerminalSymbols() != b.getNonTerminalSymbols()) {
		cout << "Nonterminal symbols" << endl;

		GrammarDiff::setDiff(a.getNonTerminalSymbols(), b.getNonTerminalSymbols());
	}

	if(a.getRules() != b.getRules()){
		cout << "Rules" << endl;

		GrammarDiff::setDiff(a.getRules(), b.getRules());
	}

	if(a.getStartSymbol() != b.getStartSymbol()){
		cout << "Start symbol" << endl;

		cout << "< " << a.getStartSymbol() << endl;
		cout << "---" << endl;
		cout << "> " << b.getStartSymbol() << endl;
	}

	if(a.getTerminalSymbols() != b.getTerminalSymbols()) {
		cout << "Terminal symbols" << endl;

		GrammarDiff::setDiff(a.getTerminalSymbols(), b.getTerminalSymbols());
	}

}

void GrammarDiff::diff(const grammar::UnknownGrammar& a, const grammar::UnknownGrammar& b) {
	if(!GrammarDiff::testDiff(a, b)) GrammarDiff::printDiff(a, b);
}
