/*
 * EpsilonClosure.cpp
 *
 *  Created on: 16. 1. 2014
 *      Author: tomas
 */

#include "EpsilonClosure.h"

using namespace alib;
using namespace automaton;
using namespace std;

namespace epsilon
{

EpsilonClosure::EpsilonClosure( const set<State> & states,
        const std::map<automaton::State, std::set<automaton::TransitionFSM>> & transitionsBySource ) :
        m_transitionsBySource( transitionsBySource ), m_states ( states )
{
    for( const auto & state : m_states )
        closure( state );
}

const set<TransitionFSM> EpsilonClosure::getTransitionsFromState ( const State & state ) const
{
    if( m_transitionsBySource.find( state ) == m_transitionsBySource.end( ) )
        return set<TransitionFSM>();

    return m_transitionsBySource.at( state );
}

void EpsilonClosure::closure( const State & state )
{
    queue<State> q;
    map<State, bool> visited;

    for( const auto & state : m_states )
        visited[ state ] = false;

    q.push( state );
    while( ! q.empty( ) )
    {
        const State & s = q.front( );
        q.pop( );
        visited[ s ] = true;
        m_closure[ state ].insert( s );

        for( const auto & transition : getTransitionsFromState( s ) )
            if( transition.getInput() == Symbol( "" ) && visited [ transition.getTo( ) ] == false )
                q.push( transition.getTo( ) );

    }
}

const set<State> & EpsilonClosure::getClosure( const State & state ) const
{
    if( m_closure.find( state ) == m_closure.end( ) )
        throw AlibException( "EpsilonClosure: Trying to get closure of state not in automata." );

    return m_closure.at( state );
}

} /* namespace epsilon */
