/*
 * RegularGrammar.h
 *
 *  Created on: Nov 23, 2013
 *      Author: martin
 */

#ifndef REGULARGRAMMAR_H_
#define REGULARGRAMMAR_H_

#include "../Grammar.h"

namespace grammar {

/**
 * Abstract class for regular grammars.
 */
class RegularGrammar: public Grammar {
protected:
	/**
	 * Checks that epsilon rule (rule that has empty right side) has start symbol
	 * on the left side and that start symbol is not present on the right side of any other rule.
	 * @param leftSide left side of the Rule
	 * @param rightSide right side of the Rule
	 * @return true when Rule is valid epsilon rule, false otherwise
	 */
	bool checkEpsilonRule(const list<Symbol>& leftSide, const list<Symbol>& rightSide) const;

	/**
	 * Checks if the Rule is epsilon rule (has empty right side).
	 * @param rule Rule to check
	 * @return true when Rule is epsioln rule, false otherwise
	 */
	bool isEpsilonRule(const Rule& rule) const;

	/**
	 * Checks if Grammar contains epsilon rule. Used when adding
	 * rule that has start symbol on the right side of the rule.
	 * @return true when grammar is epsilon free (doesn't contain epsilon rules), false otherwise
	 */
	bool containsEpsilonRule() const;
public:
	RegularGrammar();
	RegularGrammar(const set<Symbol>& nonTerminalSymbols, const set<Symbol>& terminalSymbols,
			const Symbol& startSymbol);

};

} /* namespace grammar */
#endif /* REGULARGRAMMAR_H_ */
