/*
 * TrimNFA.cpp
 *
 *  Created on: 18. 1. 2014
 *      Author: tomas
 */

#include "TrimNFA.h"

using namespace alib;
using namespace automaton;
using namespace std;
using namespace trim;

namespace trim
{

TrimNFA::TrimNFA( const FSM & fsm ) : m_origFSM( fsm )
{
    if( m_origFSM.getInitialStates( ).size( ) != 1 )
        throw AlibException( "NFA must have exactly one initial state." );
}

const FSM TrimNFA::remove( void )
{
    set<State> reachableStates = findReachableStates( );

    for( auto const & q : reachableStates )
        m_FSM.addState( q );

    for( auto const & symbol : m_origFSM.getInputAlphabet( ) )
        m_FSM.addInputSymbol( symbol );

    for( auto const & t : m_origFSM.getTransitions( ) )
        if( isInSet( t.getFrom( ), reachableStates ) )
            m_FSM.addTransition( t );


    for( auto const & state : m_origFSM.getInitialStates( ) )
        m_FSM.addInitialState( state );


    set<State> intersect;
    const set<State> & F = m_origFSM.getFinalStates( );
    set_intersection( F.begin(), F.end(), reachableStates.begin(), reachableStates.end(), inserter( intersect, intersect.begin() ) );

    for( auto const & state : intersect )
        m_FSM.addFinalState( state );

    return m_FSM;
}

const set<State> TrimNFA::findReachableStates( void ) const
{
    map<State, set<TransitionFSM>> transitionsBySourceState;

    for( auto const & t : m_origFSM.getTransitions( ) )
        transitionsBySourceState[ t.getFrom() ].insert( t );

    set<State> qprev, qcurr;
    qcurr.insert( * m_origFSM.getInitialStates().begin() );

    do
    {
        qprev = qcurr;

        for( auto const & p : qprev )
            for( auto const & transition : transitionsBySourceState[ p ] )
                qcurr.insert( transition.getTo() );
    }
    while( qcurr != qprev );

    return qcurr;
}

} /* namespace trim */
